/* othello.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define ROW_LENGTH 256

typedef int boolean;
#define TRUE  1
#define FALSE 0

#define BOARD_SIZE   8
#define BIANCO      'B'
#define NERO        'N'
#define LIBERO      '.'
#define MOVE_LENGTH  2


/* Prototipi delle procedure secondarie */

typedef char Scacchiera[BOARD_SIZE][BOARD_SIZE];

void InterpretaLineaComando (int argc, char *argv[], char *filedati);

/* Costruisce la configurazione iniziale della scacchiera S */
void InizializzaScacchiera (Scacchiera S);

/* Esegue le mosse indicate nel file filemosse */
void Othello (Scacchiera S, char *filemosse);

/* Stampa la scacchiera S */
void StampaScacchiera (Scacchiera S);

/* Stampa l'esito del gioco sulla scacchiera S */
void StampaEsito (Scacchiera S);

/* Esegue, se possibile, la mossa per il giocatore sulla scacchiera */
void EsegueMossa (char *mossa, char giocatore, Scacchiera S);

/* Verifica se la posizione corrente sulla scacchiera S e' davvero di sospensione per il giocatore */
void VerificaSospensione (Scacchiera S, char giocatore);

/* Se possibile, esegue la mossa (r,c) in direzione (dr,dc) per il giocatore sulla scacchiera;
   se non  possibile, restituisce FALSE */
boolean EsegueDirezione (int r, int c, int dr, int dc, char giocatore, Scacchiera S);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char filemosse[ROW_LENGTH];
  Scacchiera S;

  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,filemosse);

  /* Costruisce la configurazione iniziale della scacchiera */
  InizializzaScacchiera(S);

  /* Esegue le mosse indicate nel file filemosse */
  Othello(S,filemosse);

  /* Stampa la scacchiera finale */
  StampaScacchiera(S);

  /* Stampa l'esito del gioco */
  StampaEsito(S);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *filemosse)
{
  if (argc != 2)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    exit(EXIT_FAILURE);
  }

  strcpy(filemosse,argv[1]);
}


/* Costruisce la configurazione iniziale della scacchiera S */
void InizializzaScacchiera (Scacchiera S)
{
  int r, c;

  for (r = 0; r < BOARD_SIZE; r++)
    for (c = 0; c < BOARD_SIZE; c++)
      S[r][c] = LIBERO;

  S[BOARD_SIZE/2-1][BOARD_SIZE/2-1] = S[BOARD_SIZE/2][BOARD_SIZE/2] = BIANCO;
  S[BOARD_SIZE/2-1][BOARD_SIZE/2] = S[BOARD_SIZE/2][BOARD_SIZE/2-1] = NERO;
}


/* Esegue le mosse indicate nel file filemosse */
void Othello (Scacchiera S, char *filemosse)
{
  FILE *fp_filemosse;
  char mossa[MOVE_LENGTH+1];
  char giocatore;


  fp_filemosse = fopen(filemosse,"r");
  if (fp_filemosse == NULL)
  {
    fprintf(stderr,"File %s non trovato!\n",filemosse);
    exit(EXIT_FAILURE);
  }

  /* Definisce il giocatore che muove per primo */
  giocatore = NERO;

  /* Finche' ci sono mosse, ne legge una */
  while (fscanf(fp_filemosse,"%s",mossa) == 1)
  {
    /* Se e' una sospensione, valuta se davvero non ci sono mosse lecite */
    if (strcmp(mossa,"--") == 0)
      VerificaSospensione(S,giocatore);
    else
    /* Se e' una mossa regolare, valuta se e' corretta e la esegue */
      EsegueMossa(mossa,giocatore,S);

    /* Passa la mano all'altro giocatore */
    if (giocatore == BIANCO)
      giocatore = NERO;
    else
      giocatore = BIANCO;
  }

  fclose(fp_filemosse);
}


/* Stampa la scacchiera S */
void StampaScacchiera (Scacchiera S)
{
  int r, c;

  /* Stampa la cornice superiore */
  printf(" ");
  for (c = 0; c < BOARD_SIZE; c++)
    printf("%c",c+'a');
  printf("\n");

  /* Stampa la cornice laterale e la scacchiera */
  for (r = 0; r < BOARD_SIZE; r++)
  {
    printf("%d",r+1);
    for (c = 0; c < BOARD_SIZE; c++)
      printf("%c",S[r][c]);
    printf("\n");
  }
  printf("\n");
}


/* Stampa l'esito del gioco sulla scacchiera S */
void StampaEsito (Scacchiera S)
{
  int r, c;
  int nb, nn;


  /* Conta le pedine bianche e nere */
  nb = nn = 0;
  for (r = 0; r < BOARD_SIZE; r++)
    for (c = 0; c < BOARD_SIZE; c++)
      if (S[r][c] == NERO)
        nn++;
      else if (S[r][c] == BIANCO)
        nb++;

  /* Stampa il numero di pedine bianche e nere */
  printf("%d pedine nere\n",nn);
  printf("%d pedine bianche\n",nb);

  /* Stampa il risultato */
  if (nn > nb)
    printf("Vince il Nero\n");
  else if (nb > nn)
    printf("Vince il Bianco\n");
  else
    printf("Pareggio\n");
}


/* Esegue, se possibile, la mossa per il giocatore sulla scacchiera */
void EsegueMossa (char *mossa, char giocatore, Scacchiera S)
{
  boolean mossa_lecita;
  int r, c, dr, dc;


  /* Estrae dalla mossa gli indici di riga e colonna */
  r = mossa[1] - '1';
  c = mossa[0] - 'a';

  /* Verifica se la mossa e' lecita */
  mossa_lecita = FALSE;
  /* 1) la casella deve essere libera */
  if (S[r][c] == LIBERO)
    /* 2) per ogni possibile direzione */
    for (dr = -1; dr <= 1; dr++)
      for (dc = -1; dc <= 1; dc++)
        if ( (dr != 0) || (dc != 0) )
        {
          /* valuta se la direzione e' lecita e, nel caso, esegue la mossa nella direzione */
          if (EsegueDirezione(r,c,dr,dc,giocatore,S) == TRUE)
            mossa_lecita = TRUE;
        }

  /* Aggiunge la nuova pedina alla scacchiera */
  if (mossa_lecita == TRUE)
    S[r][c] = giocatore;
  else
  {
    fprintf(stderr,"La mossa %s non e' lecita per il giocatore %c!\n",mossa,giocatore);
    exit(EXIT_FAILURE);
  }
}


/* Verifica se la posizione corrente sulla scacchiera S e' davvero di sospensione per il giocatore */
void VerificaSospensione (Scacchiera S, char giocatore)
{
}


/* Se possibile, esegue la mossa (r,c) in direzione (dr,dc) per il giocatore sulla scacchiera;
   se non  possibile, restituisce FALSE */
boolean EsegueDirezione (int r, int c, int dr, int dc, char giocatore, Scacchiera S)
{
  char avversario;
  int rf, cf;

  /* Trova il colore dell'avversario */
  if (giocatore == BIANCO)
    avversario = NERO;
  else
    avversario = BIANCO;

  /* Trova la casella iniziale in direzione (dr,dc) */
  rf = r+dr;
  cf = c+dc;

  /* Si sposta nella casella adiacente finche' e' nella scacchiera e occupata dall'avversario */
  while ( (0 <= rf) && (rf < BOARD_SIZE) && (0 <= cf) && (cf < BOARD_SIZE) && (S[rf][cf] == avversario) )
  {
    rf += dr;
    cf += dc;
  }

  /* La mossa e' vietata se la casella finale e' fuori dalla scacchiera,
     o non e' occupata dal giocatore o infine e' adiacente a quella iniziale */
  if ( (rf < 0) || (rf >= BOARD_SIZE) || (cf < 0) || (cf >= BOARD_SIZE) ||
       (S[rf][cf] != giocatore) || ( (rf == r+dr) && (cf == c+dc) ) )
     return FALSE;
  else
  {
    /* Se la mossa e' lecita, torna dalla casella finale a quella iniziale marcando le celle */
    while ( (rf != r) || (cf != c) )
    {
      S[rf][cf] = giocatore;
      rf -= dr;
      cf -= dc;
    }

    return TRUE;
  }
}
